#include "p24hj128gp502.h"

#include "adcDrv2.h"
//#include "tglPin.h"
#include "UART.h"
#include <string.h>
#include <stdio.h>


#define NUMSAMP	256
int BufferA[NUMSAMP] __attribute__((space(dma)));
int BufferB[NUMSAMP] __attribute__((space(dma)));

void ProcessADCSamples(int AdcBuffer);

//Functions:
//initAdc1() is used to configure A/D to convert AIN0 using CH0 and CH1 sample/hold in sequencial mode 
//at 1.1MHz throughput rate. ADC clock is configured at 13.3Mhz or Tad=75ns
void initAdc1(void)
{


		AD1CON1bits.FORM   = 3;		// Data Output Format: Signed Fraction (Q15 format)
		AD1CON1bits.SSRC   = 7;		// Interan Counter (SAMC) ends sampling and starts convertion
		AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
		AD1CON1bits.AD12B  = 0;		// 10-bit ADC operation

		AD1CON2bits.CHPS  = 1;		// Converts CH0/CH1
     
		AD1CON3bits.ADRC=0;			// ADC Clock is derived from Systems Clock
		AD1CON3bits.SAMC=31; 		// Auto Sample Time = 0*Tad		
		AD1CON3bits.ADCS=255;		    // ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*3 = 75ns (13.3Mhz)
									// ADC Conversion Time for 10-bit Tc=12*Tab =  900ns (1.1MHz)
					
		AD1CON1bits.ADDMABM = 1; 	// DMA buffers are built in conversion order mode
		AD1CON2bits.SMPI    = 0;	// SMPI must be 0


  	//AD1CHS0/AD1CHS123: A/D Input Select Register
        AD1CHS0bits.CH0SA=0;		// MUXA +ve input selection (AIN0) for CH0
		AD1CHS0bits.CH0NA=0;		// MUXA -ve input selection (Vref-) for CH0

        AD1CHS123bits.CH123SA=0;	// MUXA +ve input selection (AIN0) for CH1
		AD1CHS123bits.CH123NA=0;	// MUXA -ve input selection (Vref-) for CH1


	//AD1PCFGH/AD1PCFGL: Port Configuration Register
		AD1PCFGL=0xFFFF;
		//AD1PCFGH=0xFFFF;
		AD1PCFGLbits.PCFG0 = 0;		// AN0 as Analog Input


        IFS0bits.AD1IF = 0;			// Clear the A/D interrupt flag bit
        IEC0bits.AD1IE = 0;			// Do Not Enable A/D interrupt 
        AD1CON1bits.ADON = 1;		// Turn on the A/D converter	
    
        //tglPinInit();

}

// DMA0 configuration
// Direction: Read from peripheral address 0-x300 (ADC1BUF0) and write to DMA RAM 
// AMODE: Register indirect with post increment
// MODE: Continuous, Ping-Pong Mode
// IRQ: ADC Interrupt
// ADC stores results stored alternatively between DMA_BASE[0]/DMA_BASE[16] on every 16th DMA request 

void initDma0(void)
{
	DMA0CONbits.AMODE = 0;			// Configure DMA for Register indirect with post increment
	DMA0CONbits.MODE  = 2;			// Configure DMA for Continuous Ping-Pong mode

	DMA0PAD=(int)&ADC1BUF0;
	DMA0CNT=(NUMSAMP-1);				
	
	DMA0REQ=13;	
	
	DMA0STA = __builtin_dmaoffset(BufferA);		
	DMA0STB = __builtin_dmaoffset(BufferB);

	IFS0bits.DMA0IF = 0;			//Clear the DMA interrupt flag bit
    IEC0bits.DMA0IE = 1;			//Set the DMA interrupt enable bit
	
	DMA0CONbits.CHEN=1;


}





/*=============================================================================
_DMA0Interrupt(): ISR name is chosen from the device linker script.
=============================================================================*/
unsigned int DmaBuffer = 0;

void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{

		if(DmaBuffer == 0)
		{
			ProcessADCSamples(*BufferA);
		}
		else
		{
			ProcessADCSamples(*BufferB);

		}

		DmaBuffer ^= 1;

		//tglPin();					// Toggle RA6	
        IFS0bits.DMA0IF = 0;		//Clear the DMA0 Interrupt Flag
}



void ProcessADCSamples(int AdcBuffer)
{
	//void TX2 (char *strTX);
	
	char strADC[100];
	sprintf(strADC," The buffer is %d)", AdcBuffer);
	TX2(strADC);
}




